EXPORTS
    Init_gtk3
    rbgtk_initialize_gtkobject
    gtk_accel_group_entry_get_type
    gtk_accel_key_get_type
    gtk_allocation_get_type
    gtk_bindingset_get_type
    gtk_recent_data_get_type
    gtk_recent_filter_info_get_type
    gtk_text_appearance_get_type
